(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
typeof define === 'function' && define.amd ? define(['exports'], factory) :
(global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.MonacoJSXHighlighter = {}));
})(this, (function (exports) { 'use strict';

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;
  if (!css || typeof document === 'undefined') { return; }
  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';
  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }
  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".JSXElement.JSXIdentifier{color:#4169e1}.JSXElement.JSXBracket{color:#ff8c00}.JSXElement.JSXText{color:#b8860b}.JSXElement.JSXGlyph{background:cyan;opacity:.25}.JSXClosingFragment.JSXBracket,.JSXOpeningElement.JSXBracket,.JSXOpeningFragment.JSXBracket{color:#ff8c00;font-weight:700}.JSXOpeningElement.JSXIdentifier{color:#4169e1}.JSXClosingElement.JSXBracket{color:#ff8c00;font-weight:lighter}.JSXClosingElement.JSXIdentifier{color:#4169e1;font-weight:lighter}.JSXAttribute.JSXIdentifier{color:#4682b4}.JSXExpressionContainer.JSXBracket,.JSXSpreadAttribute.JSXBracket,.JSXSpreadChild.JSXBracket{color:#ff8c00}";
styleInject(css_248z);

function configureLoc2Range(monaco, parserType = 'babel') {
  switch (parserType) {
    case 'babel':
    default:
      return (loc, startLineOffset = 0, startColumnOffset = 0, endLineOffset = 0, endColumnOffset = 0) => {
        if (!loc || !loc.start) {
          return new monaco.Range(1, 1, 1, 1);
        }
        return new monaco.Range(startLineOffset + loc.start.line, startColumnOffset + loc.start.column + 1, endLineOffset + loc.end ? loc.end.line : loc.start.line, endColumnOffset + loc.end ? loc.end.column + 1 : loc.start.column + 1);
      };
  }
}
function configureRange2Loc(parserType = 'babel') {
  switch (parserType) {
    case 'babel':
    default:
      return (rangeOrPosition, startLineOffset = 0, startColumnOffset = 0, endLineOffset = 0, endColumnOffset = 0) => {
        const loc = {
          start: {
            line: 0,
            column: 0
          },
          end: {
            line: 0,
            column: 0
          }
        };
        if (!rangeOrPosition) {
          return loc;
        }
        if (rangeOrPosition.lineNumber) {
          loc.start.line = startLineOffset + rangeOrPosition.lineNumber;
          loc.start.column = startColumnOffset + rangeOrPosition.column - 1;
          loc.end.line = endLineOffset + rangeOrPosition.lineNumber;
          loc.end.column = endColumnOffset + rangeOrPosition.column - 1;
        } else {
          loc.start.line = startLineOffset + rangeOrPosition.startLineNumber;
          loc.start.column = startColumnOffset + rangeOrPosition.startColumn - 1;
          loc.end.line = endLineOffset + rangeOrPosition.endLineNumber;
          loc.end.column = endColumnOffset + rangeOrPosition.endColumn - 1;
        }
        return loc;
      };
  }
}
class MonacoEditorManager {
  static COMMENT_ACTION_ID = "editor.action.commentLine";
  constructor(monacoEditor, monaco, parserType = 'babel') {
    this.monacoEditor = monacoEditor;
    this.monaco = monaco;
    this.parserType = parserType;
    this.loc2Range = configureLoc2Range(monaco, parserType);
    this.range2Loc = configureRange2Loc(parserType);
  }
  runEditorCommentLineAction = () => {
    return this.monacoEditor.getAction(MonacoEditorManager.COMMENT_ACTION_ID).run();
  };
  getLineIndentationColumn = lineNumber => {
    return this.monacoEditor.getModel().getLineFirstNonWhitespaceColumn(lineNumber);
  };
  getCommentableStartingRange = range => {
    const startColumn = this.getLineIndentationColumn(range.startLineNumber);
    const commentableRange = new this.monaco.Range(range.startLineNumber, startColumn, range.startLineNumber, startColumn);
    return commentableRange;
  };
  getCommentContainingStartingRange = range => {
    let startColumn = this.getLineIndentationColumn(range.startLineNumber);
    startColumn = startColumn ? startColumn - 1 : 0;
    const containingRange = new this.monaco.Range(range.startLineNumber, startColumn, range.startLineNumber, startColumn);
    return containingRange;
  };
  getSelectionFirstLineText = () => {
    const model = this.monacoEditor.getModel();
    const {
      startLineNumber
    } = this.monacoEditor.getSelection();
    const jsCommentRange = new this.monaco.Range(startLineNumber, this.getLineIndentationColumn(startLineNumber), startLineNumber, model.getLineMaxColumn(startLineNumber));
    return model.getValueInRange(jsCommentRange);
  };
}

class JSXExpressionCollector {
  constructor(ast, traverse, traverseOptions = {}) {
    const jsxExpressions = [];
    const enter = path => {
      if (path.type.toUpperCase().includes("JSX")) {
        jsxExpressions.push(path);
      }
    };
    traverse(ast, { ...traverseOptions,
      enter
    });
    this.all = () => {
      return [...jsxExpressions];
    };
    this.filterByType = (type = 'JSXElement') => {
      return jsxExpressions.filter(path => path.type === type);
    };
  }
}
class BabelDataExtractor {
  static isJSXIdentifier(path) {
    return path && (path.key === 'object' || path.key === 'property' || path.key === 'name' || path.key === 'namespace');
  }
  static isParentJSXAttribute(path) {
    return path && path.parentPath && path.parentPath.isJSXAttribute();
  }
  static getLoc(path) {
    return path && path.node && path.node.loc;
  }
  static cloneLoc(path) {
    const loc = BabelDataExtractor.getLoc(path);
    if (!loc) {
      return null;
    }
    return {
      start: { ...loc.start
      },
      end: { ...loc.end
      }
    };
  }
  static getCuratedLoc(path) {
    const loc = BabelDataExtractor.cloneLoc(path);
    if (!loc) {
      return [null, null, null, null];
    }
    if (path.key === 'object' && path.container) {
      loc.end = { ...path.container.property.loc.start
      };
    }
    return loc;
  }
  static extractJSXOpeningElement(path) {
    const openingElement = path.node.openingElement;
    if (!openingElement) {
      return [null, null, null, null];
    }
    const elementName = openingElement.name.name;
    const startLoc = {
      start: { ...openingElement.loc.start
      },
      end: { ...openingElement.name.loc.start
      }
    };
    const endLoc = {
      start: { ...openingElement.loc.end
      },
      end: { ...openingElement.loc.end
      }
    };
    endLoc.start.column--;
    if (openingElement.selfClosing) {
      endLoc.start.column--;
    }
    return [openingElement, elementName, startLoc, endLoc];
  }
  static extractJSXClosingElement(path) {
    const closingElement = path.node.closingElement;
    if (!closingElement) {
      return [null, null, null, null];
    }
    const elementName = closingElement.name && closingElement.name.name;
    const startLoc = {
      start: { ...closingElement.loc.start
      },
      end: { ...closingElement.name.loc.start
      }
    };
    const endLoc = {
      start: { ...closingElement.loc.end
      },
      end: { ...closingElement.loc.end
      }
    };
    endLoc.start.column--;
    return [closingElement, elementName, startLoc, endLoc];
  }
  static extractJSXExpressionEdges(path) {
    const loc = BabelDataExtractor.getLoc(path);
    if (!loc) {
      return [null, null, null, null];
    }
    let innerNode = null;
    let innerLocKey = path.isJSXSpreadChild() ? 'expression' : path.isJSXSpreadAttribute() ? 'argument' : null;
    let innerLoc = null;
    if (innerLocKey) {
      innerNode = path.node[innerLocKey];
      innerLoc = {
        start: { ...innerNode.loc.start
        },
        end: { ...innerNode.loc.end
        }
      };
      if (innerNode.extra && innerNode.extra.parenthesized) {
        innerLoc.start.column--;
        innerLoc.end.column++;
      }
    } else {
      innerLoc = {
        start: { ...loc.start
        },
        end: { ...loc.end
        }
      };
      innerLoc.start.column++;
      innerLoc.end.column--;
    }
    const startEdgeLoc = {
      start: { ...loc.start
      },
      end: { ...innerLoc.start
      }
    };
    const endEdgeLoc = {
      start: { ...innerLoc.end
      },
      end: { ...loc.end
      }
    };
    return [innerNode, innerLocKey, startEdgeLoc, endEdgeLoc];
  }
}

function prepareOptions(path, jsxTypeOptions = {}, highlighterOptions = {}) {
  return highlighterOptions.iShowHover ? { ...jsxTypeOptions,
    ...{
      hoverMessage: `(${path.type})`
    }
  } : jsxTypeOptions;
}
const HIGHLIGHT_TYPE = {
  ELEMENT: 'ELEMENT',
  ALL: 'ALL',
  IDENTIFIER: 'IDENTIFIER',
  EDGE: 'EDGE',
  STYLE: 'STYLE'
};
const HIGHLIGHT_MODE = {
  [HIGHLIGHT_TYPE.ELEMENT]: (path, jsxTypeOptions, highlighterOptions) => {
    const [openingElement, elementName, startLoc, endLoc] = BabelDataExtractor.extractJSXOpeningElement(path);
    const result = [];
    if (openingElement) {
      result.push([startLoc, highlighterOptions.isUseSeparateElementStyles ? JSXTypes.JSXBracket.openingElementOptions : JSXTypes.JSXBracket.options]);
      result.push([endLoc, highlighterOptions.isUseSeparateElementStyles ? JSXTypes.JSXBracket.openingElementOptions : JSXTypes.JSXBracket.options]);
    }
    const [closingElement,, closingElementStartLoc, closingElementEndLoc] = BabelDataExtractor.extractJSXClosingElement(path);
    if (closingElement) {
      result.push([closingElementStartLoc, highlighterOptions.isUseSeparateElementStyles ? JSXTypes.JSXBracket.closingElementOptions : JSXTypes.JSXBracket.options]);
      result.push([closingElementEndLoc, highlighterOptions.isUseSeparateElementStyles ? JSXTypes.JSXBracket.closingElementOptions : JSXTypes.JSXBracket.options]);
    }
    const loc = BabelDataExtractor.getLoc(path);
    highlighterOptions.isHighlightGlyph && result.push([loc, JSXTypes.JSXElement.options(elementName)]);
    return result;
  },
  [HIGHLIGHT_TYPE.ALL]: (path, jsxTypeOptions, highlighterOptions) => {
    const curatedLoc = BabelDataExtractor.getCuratedLoc(path);
    const result = [];
    curatedLoc && result.push([curatedLoc, prepareOptions(path, jsxTypeOptions, highlighterOptions)]);
    return result;
  },
  [HIGHLIGHT_TYPE.IDENTIFIER]: (path, jsxTypeOptions, highlighterOptions) => {
    if (!BabelDataExtractor.isJSXIdentifier(path)) {
      return [];
    }
    return HIGHLIGHT_MODE[HIGHLIGHT_TYPE.ALL](path, BabelDataExtractor.isParentJSXAttribute(path) ? JSXTypes.JSXAttribute.options : jsxTypeOptions, highlighterOptions);
  },
  [HIGHLIGHT_TYPE.EDGE]: (path, jsxTypeOptions, highlighterOptions) => {
    const options = prepareOptions(path, jsxTypeOptions, highlighterOptions);
    const [,, startEdgeLoc, endEdgeLoc] = BabelDataExtractor.extractJSXExpressionEdges(path);
    const result = [];
    result.push([startEdgeLoc, options]);
    result.push([endEdgeLoc, options]);
    return result;
  },
  [HIGHLIGHT_TYPE.STYLE]: () => []
};
const JSXTypes = {
  JSXIdentifier: {
    highlightScope: HIGHLIGHT_TYPE.IDENTIFIER,
    options: {
      inlineClassName: 'JSXElement.JSXIdentifier'
    }
  },
  JSXOpeningFragment: {
    highlightScope: HIGHLIGHT_TYPE.ALL,
    options: {
      inlineClassName: 'JSXOpeningFragment.JSXBracket'
    }
  },
  JSXClosingFragment: {
    highlightScope: HIGHLIGHT_TYPE.ALL,
    options: {
      inlineClassName: 'JSXClosingFragment.JSXBracket'
    }
  },
  JSXText: {
    highlightScope: HIGHLIGHT_TYPE.ALL,
    options: {
      inlineClassName: 'JSXElement.JSXText'
    }
  },
  JSXExpressionContainer: {
    highlightScope: HIGHLIGHT_TYPE.EDGE,
    options: {
      inlineClassName: 'JSXExpressionContainer.JSXBracket'
    }
  },
  JSXSpreadChild: {
    highlightScope: HIGHLIGHT_TYPE.EDGE,
    options: {
      inlineClassName: 'JSXSpreadChild.JSXBracket'
    }
  },
  JSXSpreadAttribute: {
    highlightScope: HIGHLIGHT_TYPE.EDGE,
    options: {
      inlineClassName: 'JSXSpreadAttribute.JSXBracket'
    }
  },
  JSXElement: {
    highlightScope: HIGHLIGHT_TYPE.ELEMENT,
    options: elementName => ({
      glyphMarginClassName: 'JSXElement.JSXGlyph',
      glyphMarginHoverMessage: `JSX Element${elementName ? ': ' + elementName : ''}`
    })
  },
  JSXBracket: {
    highlightScope: HIGHLIGHT_TYPE.STYLE,
    options: {
      inlineClassName: 'JSXElement.JSXBracket'
    },
    openingElementOptions: {
      inlineClassName: 'JSXOpeningElement.JSXBracket'
    },
    closingElementOptions: {
      inlineClassName: 'JSXClosingElement.JSXBracket'
    }
  },
  JSXOpeningElement: {
    highlightScope: HIGHLIGHT_TYPE.STYLE,
    options: {
      inlineClassName: 'JSXOpeningElement.JSXIdentifier'
    }
  },
  JSXClosingElement: {
    highlightScope: HIGHLIGHT_TYPE.STYLE,
    options: {
      inlineClassName: 'JSXClosingElement.JSXIdentifier'
    }
  },
  JSXAttribute: {
    highlightScope: HIGHLIGHT_TYPE.STYLE,
    options: {
      inlineClassName: 'JSXAttribute.JSXIdentifier'
    }
  }
};
class DecoratorMapper {
  constructor(monacoEditor, loc2Range, _JSXTypes = JSXTypes) {
    let decorators = [];
    let jsxDecoratorIds = [];
    const addDecorator = ([loc, options]) => {
      return decorators.push({
        range: loc2Range(loc),
        options
      });
    };
    const deltaDecorations = () => {
      console.log("deltaDecorations", decorators.length, jsxDecoratorIds.length);
      jsxDecoratorIds = monacoEditor.deltaDecorations(jsxDecoratorIds || [], decorators);
      return jsxDecoratorIds;
    };
    this.deltaJSXDecorations = (jsxExpressionCollector, options) => {
      for (const jsxType in _JSXTypes) {
        jsxExpressionCollector.filterByType(jsxType).forEach(path => HIGHLIGHT_MODE[_JSXTypes[jsxType].highlightScope](path, _JSXTypes[jsxType].options, options).forEach(entry => addDecorator(entry)));
      }
      return deltaDecorations();
    };
    this.reset = () => {
      decorators = [];
      deltaDecorations();
    };
    this.reset();
  }
}

const JSXCommentContexts = {
  JS: 'JS',
  JSX: 'JSX'
};
function getJSXContext(jsxExpressions, commentableRange, commentContainingRange, loc2Range) {
  if (!(jsxExpressions && commentableRange && commentContainingRange && loc2Range)) {
    return JSXCommentContexts.JS;
  }
  let minRange = null;
  let minCommentableRange = null;
  let path = null;
  let commentablePath = null;
  jsxExpressions.forEach(p => {
    const jsxRange = loc2Range(p.node.loc);
    if ((p.key === 'name' || p.key === 'property') && p.isJSXIdentifier() && jsxRange.intersectRanges(commentableRange)) {
      if (!minCommentableRange || minCommentableRange.containsRange(jsxRange)) {
        minCommentableRange = jsxRange;
        commentablePath = p;
      }
    }
    if (jsxRange.intersectRanges(commentContainingRange)) {
      if (!minRange || minRange.containsRange(jsxRange)) {
        minRange = jsxRange;
        path = p;
      }
    }
  });
  if (!path || path.isJSXExpressionContainer() || commentablePath) {
    return JSXCommentContexts.JS;
  } else {
    return JSXCommentContexts.JSX;
  }
}
class Commenter {
  constructor(monacoEditorManager, parseJSXExpressionsPromise) {
    this.monacoEditorManager = monacoEditorManager;
    this.parseJSXExpressionsPromise = parseJSXExpressionsPromise;
    this._editorCommandId = null;
    this._isJSXCommentCommandActive = false;
  }
  editorCommandOnDispose = () => {
    this._isJSXCommentCommandActive = false;
  };
  runJsxCommentAction = (selection, commentContext) => {
    const {
      monacoEditorManager
    } = this;
    const {
      monacoEditor,
      monaco,
      runEditorCommentLineAction,
      getSelectionFirstLineText
    } = monacoEditorManager;
    const jsCommentText = getSelectionFirstLineText();
    if (jsCommentText.match(/^\s*\/[/*]/)) {
      runEditorCommentLineAction(monacoEditor);
      return;
    }
    const model = monacoEditor.getModel();
    let isUnCommentAction = true;
    const commentsData = [];
    for (let i = selection.startLineNumber; i <= selection.endLineNumber; i++) {
      const commentRange = new monaco.Range(i, model.getLineFirstNonWhitespaceColumn(i), i, model.getLineMaxColumn(i));
      const commentText = model.getValueInRange(commentRange);
      commentsData.push({
        commentRange,
        commentText
      });
      isUnCommentAction = isUnCommentAction && !!commentText.match(/{\/\*/);
    }
    if (commentContext !== JSXCommentContexts.JSX && !isUnCommentAction) {
      runEditorCommentLineAction(monacoEditor);
      return;
    }
    let editOperations = [];
    let commentsDataIndex = 0;
    for (let i = selection.startLineNumber; i <= selection.endLineNumber; i++) {
      let {
        commentText,
        commentRange
      } = commentsData[commentsDataIndex++];
      if (isUnCommentAction) {
        commentText = commentText.replace(/{\/\*/, '');
        commentText = commentText.replace(/\*\/}/, '');
      } else {
        commentText = `{/*${commentText}*/}`;
      }
      editOperations.push({
        identifier: {
          major: 1,
          minor: 1
        },
        range: commentRange,
        text: commentText,
        forceMoveMarkers: true
      });
    }
    editOperations.length && monacoEditor.executeEdits(this._editorCommandId, editOperations);
  };
  addJSXCommentCommand = () => {
    const {
      monacoEditorManager,
      parseJSXExpressionsPromise
    } = this;
    const {
      monacoEditor,
      monaco,
      loc2Range,
      runEditorCommentLineAction,
      getCommentableStartingRange,
      getCommentContainingStartingRange
    } = monacoEditorManager;
    if (this._editorCommandId) {
      this._isJSXCommentCommandActive = true;
      return this.editorCommandOnDispose;
    }
    this._editorCommandId = monacoEditor.addCommand(monaco.KeyMod.CtrlCmd | monaco.KeyCode.US_SLASH, () => {
      if (!this._isJSXCommentCommandActive) {
        return runEditorCommentLineAction(monacoEditor);
      }
      parseJSXExpressionsPromise().then(jsxExpressions => {
        const selection = monacoEditor.getSelection();
        const commentContext = getJSXContext(jsxExpressions, getCommentableStartingRange(selection), getCommentContainingStartingRange(selection), loc2Range);
        this.runJsxCommentAction(selection, commentContext);
      });
    });
    this._isJSXCommentCommandActive = true;
    monacoEditor.onDidDispose(this.editorCommandOnDispose);
    return this.editorCommandOnDispose;
  };
}

const defaultOptions = {
  parser: 'babel',
  isHighlightGlyph: false,
  iShowHover: false,
  isUseSeparateElementStyles: false,
  monacoEditorManager: null,
  decoratorMapper: null,
  commenter: null
};
class MonacoJSXHighlighter {
  constructor(monaco, parse, traverse, monacoEditor, options = {}) {
    this.options = { ...defaultOptions,
      ...options
    };
    const {
      parserType,
      monacoEditorManager,
      decoratorMapper,
      commenter
    } = this.options;
    this.monacoEditor = monacoEditor;
    this.monacoEditorManager = monacoEditorManager || new MonacoEditorManager(monacoEditor, monaco, parserType);
    this.decoratorMapper = decoratorMapper || new DecoratorMapper(monacoEditor, this.monacoEditorManager.loc2Range);
    this._isHighlightBoundToModelContentChanges = false;
    this.parseJSXExpressionsPromise = () => {
      return new Promise(resolve => {
        this.getAstPromise(false).then(ast => {
          resolve(new JSXExpressionCollector(ast, traverse).all());
        });
      });
    };
    this.commenter = commenter || new Commenter(this.monacoEditorManager, this.parseJSXExpressionsPromise);
    this.addJSXCommentCommand = this.commenter.addJSXCommentCommand;
    let ast = null;
    this.getAstPromise = () => {
      return new Promise((resolve, reject) => {
        const editorValue = monacoEditor.getValue();
        try {
          ast = parse(editorValue);
          resolve(ast);
        } catch (e) {
          reject(e);
        }
      });
    };
    this.highlight = ast => {
      return new Promise((resolve, reject) => {
        if (!ast) {
          resolve({
            ast,
            jsxExpressionCollector: null
          });
        }
        try {
          const jsxExpressionCollector = new JSXExpressionCollector(ast, traverse);
          this.decoratorMapper.deltaJSXDecorations(jsxExpressionCollector, this.options);
          resolve({
            ast,
            jsxExpressionCollector
          });
        } catch (e) {
          reject(e);
        }
      });
    };
    this.highlightCode = (afterHighlight = ast => ast, onError = error => error, getAstPromise = this.getAstPromise, onJsParserErrors = error => error) => {
      return getAstPromise().then(ast => {
        this.highlight(ast).then(afterHighlight).catch(onError);
      }).catch(onJsParserErrors);
    };
  }
  highlightOnDidChangeModelContent = (debounceTime = 100, afterHighlight = ast => ast, onHighlightError = error => error, getAstPromise = this.getAstPromise, onParseAstError = error => error) => {
    const highlightCallback = () => {
      return this.highlightCode(afterHighlight, onHighlightError, getAstPromise, onParseAstError);
    };
    highlightCallback();
    let tid = null;
    let highlighterDisposer = {
      onDidChangeModelContentDisposer: this.monacoEditor.onDidChangeModelContent(() => {
        clearTimeout(tid);
        tid = setTimeout(highlightCallback, debounceTime);
      }),
      onDidChangeModelDisposer: this.monacoEditor.onDidChangeModel(() => {
        highlightCallback();
      })
    };
    highlighterDisposer.dispose = () => {
      highlighterDisposer.onDidChangeModelContentDisposer.dispose();
      highlighterDisposer.onDidChangeModelDisposer.dispose();
    };
    this._isHighlightBoundToModelContentChanges = true;
    const onDispose = () => {
      this.decoratorMapper.reset();
      if (!this._isHighlightBoundToModelContentChanges) {
        return;
      }
      this._isHighlightBoundToModelContentChanges = false;
      highlighterDisposer && highlighterDisposer.dispose();
      highlighterDisposer = null;
    };
    this.monacoEditor.onDidDispose(() => {
      this.decoratorMapper.reset();
      highlighterDisposer = null;
      this._isHighlightBoundToModelContentChanges = false;
    });
    return onDispose;
  };
}

exports.HIGHLIGHT_MODE = HIGHLIGHT_MODE;
exports.HIGHLIGHT_TYPE = HIGHLIGHT_TYPE;
exports.JSXTypes = JSXTypes;
exports["default"] = MonacoJSXHighlighter;
exports.prepareOptions = prepareOptions;

Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=monaco-jsx-highlighter.js.map
